/*!
 @header AssertLog
 @abstract Macro definitions for asserts
 @copyright 2008 MYOB Technology Pty Ltd., all rights reserved.
 @updated 2008-07-16
 @discussion Macros used to assert a condition. To be used when the assert should only be performed in a debug build.
 
 This header uses the DEBUG macro to enable additional debugging. Make sure you define DEBUG in the debug build project settings in Xcode.
*/

#pragma once

#ifdef _DEBUG
/*!
 @defined 
 @abstract   Make sure a condition is true
 @param      e the condition to test.
 @discussion Will test a condition and on false will output to the console log, will break program execution. Only works in debug build when _DEBUG defined.
 This was named with "CP" prefix so that it avoids the generic ASSERT macro that is defined so often.
 Output will look like: Failed Assert! ("e")    Function:-[mainWindowController awakeFromNib] File:mainWindowController.m Line:31
 */
#ifdef _MAC
#define CPASSERT(e) ((void) ((e) ? 0 : ((void)NSLog((@"Failed Assert! (%s);    Function:%s File:%@ Line:%d"), #e, __func__, [@__FILE__ lastPathComponent], __LINE__), assert(e), 0)))
#else  // #ifdef _MAC
#include <assert.h>
#define CPASSERT(e) ((void) ((e) ? 0 : (DebugLog("Failed Assert! (%s) - %s    Function:%s File:%s Line:%d", #e, __FUNCTION__, __FILE__, __LINE__), assert(e), 0)))
#endif  // #else  // #ifdef _MAC

/*!
 @defined 
 @abstract   Make sure a condition is true
 @param      e the condition to test.
 @discussion Will test a condition and on false will output to the console log, will NOT break program execution. Only works in debug build when DEBUG defined.
 
 Output will look like: Failed Assert! ("e")    Function:-[mainWindowController awakeFromNib] File:mainWindowController.m Line:31
 */
#ifdef _MAC
#define ASSERT_LOG(e)  ((void) ((e) ? 0 : ((void)NSLog((@"Failed Assert! (%s);    Function:%s File:%@ Line:%d"), #e, __func__, [@__FILE__ lastPathComponent], __LINE__), 0)))
#else  // #ifdef _MAC
#define ASSERT_LOG(e) ((void) ((e) ? 0 : (DebugLog("Failed Assert! (%s)  Function:%s File:%s Line:%d", #e, __FUNCTION__, __FILE__, __LINE__), 0)))
#endif  // #else  // #ifdef _MAC
/*!
 @defined 
 @abstract   Make sure a condition is true
 @param      e the condition to test.
 @param      m the message to add to the log output.
 @discussion Will test a condition and on false will output to the console log, will NOT break program execution. Allows you to put an addition message in the
 console output. Only works in debug build when DEBUG defined.
 
 Output will look like: Failed Assert! ("e") - "m"    Function:-[mainWindowController awakeFromNib] File:mainWindowController.m Line:31
 */
#ifdef _MAC
#define ASSERT_LOG_WITH_MESSAGE(e,m) ((void) ((e) ? 0 : ((void)NSLog((@"Failed Assert! (%s) - %@    Function:%s File:%@ Line:%d"), #e, m, __func__, [@__FILE__ lastPathComponent], __LINE__), 0)))
#else  // #ifdef _MAC
#define ASSERT_LOG_WITH_MESSAGE(e,m) ((void) ((e) ? 0 : (DebugLog("Failed Assert! (%s) - %s    Function:%s File:%s Line:%d", #e, m, __FUNCTION__, __FILE__, __LINE__), 0)))
#endif  // #else  // #ifdef _MAC

#else  // #ifdef _DEBUG

#define CPASSERT(e)
#define ASSERT_LOG(e)
#define ASSERT_LOG_WITH_MESSAGE(e,m)
#endif  // #else  // #ifdef _DEBUG
