////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPDate.h
//    Cross-platform date class.
//
//  Copyright (C) 2009-2010 MYOB US, Inc., all rights reserved.
//
//  Change History (most recent first):
//
//  06/09/10  G.R.W.  Added method DateFromMonthDayYearHourMinuteSecond and SetFromMonthDayYearHourMinuteSecond.
//  03/25/09  K.L.S.  Created file.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
 @header CPDate
 CPDate objects represent a single point in time.
 */

#pragma once

#include "CrossPlatform/CPObject.h"
#include "CrossPlatform/CPString.h"

/*!
 @class CPDate
 @abstract CPDate represent a single point in time.
 CPDate is a class that provides behavior for creating dates, comparing dates, representing dates, computing intervals, and similar
 functionality. CPDate presents a programmatic interface through which suitable date objects are requested and returned. Date objects
 returned from CPDate are lightweight and immutable since they represent an invariant point in time. This class is designed to provide the
 foundation for arbitrary calendrical representations.
 
 The sole primitive method of CPDate, timeIntervalSinceReferenceDate, provides the basis for all the other methods in the CPDate interface.
 This method returns a time value relative to an absolute reference datethe first instant of 1 January 2001, GMT.
 
 To parse strings containing dates and to generate string representations of a date, you should use an instance of CPDateFormatter using the
 methods DateFromString() and StringFromDate() respectivelysee CPDateFormatter documentation for more details.
 
 Note: CPDate uses NSDate or its Core Foundation counterpart, CFDate Reference to provide most of the functionality. NSDate and CFDate are
 bridged Foundation objects. Therefore, in a method where you see an NSDate* parameter, you can pass a CFDateRef, and in a function where
 you see a CFDateRef parameter, you can pass an NSDate instance. NSDate is currently not available on Windows.
 
 For more information see the documentation for NSDate or CFDateRef:
 
 http://developer.apple.com/mac/library/documentation/Cocoa/Reference/Foundation/Classes/NSDate_Class/Reference/Reference.html
 http://developer.apple.com/mac/library/documentation/CoreFoundation/Reference/CFDateRef/Reference/reference.html
 */

typedef double CPTimeInterval;

class EXTERNAL_API CPDate : public CPObject
{
public:
	/*!
	 @method     ClassType (class method)
	 */
	static CPString ClassType(void);
	
/*!
 @method     DistantFuture (class method)
 @abstract   Returns an CPDate object representing a date in the distant future.
 @discussion You can pass this value when an CPDate object is required to have the date argument essentially ignored.
 For example, in a function that takes a date range you can use this date to make the upper limit of the range open ended.
 @result     A CPDate object representing a date in the distant future (in terms of centuries).
 */
	static const CPDate& DistantFuture(void);
	
	/*!
	 @method     DistantPast (class method)
	 @abstract   Returns an CPDate object representing a date in the distant past.
	 @discussion You can use this object as a control date, a guaranteed temporal boundary.
	 @result     A CPDate object representing a date in the distant past (in terms of centuries).
	 */
	static const CPDate& DistantPast(void);
	
	/*!
	 @method     Today (class method)
	 @abstract   Returns an CPDate object representing the current date at 12 midnight using the current time zone.
	 @discussion You can use this object when you want a date and do not want any time components. The time components are all zero.
	 @result     A CPDate object representing the current date at 12 midnight using the current time zone.
	 */
	static CPDate Today(void);
	
	/*!
	 @method     Now (class method)
	 @abstract   Returns an CPDate object representing the current date and time.
	 @discussion You can use this object when you want the current date and time.
	 @result     A CPDate object representing the current date and time using the current time zone.
	 */
	static CPDate Now(void);
	
	/*!
	 @method     ReferenceDate (class method)
	 @abstract   Returns an CPDate object representing the reference date of 1 January 2001, GMT.
	 @discussion You can use this object when you want a date and do not want any time components. The time components are all zero.
	 @result     A CPDate object representing the reference date.
	 */
	static const CPDate& ReferenceDate(void);
	
			
	static CPDate DateWithTimeIntervalSince1970(double Seconds);
	static CPDate DateWithTimeIntervalSinceReferenceDate(double Seconds);
	static CPDate DateWithTimeIntervalSinceNow(double Seconds);

	/*!
	 @method     DateFromMonthDayYear (class method)
	 @abstract   Returns an CPDate object representing the date specified by the month, day and year.
	 @discussion You can use this object when you want a specific date and do not want any time components. The time components are all zero.
	 @param      Month  The month, identified as a number from 1 to 12.
	 @param      Day  The day of the month.
	 @param      Year  The year, identifed as a complete number (includes century).
	 @result     A CPDate object representing the date specified.
	 */
	static CPDate DateFromMonthDayYear(int Month, int Day, int Year);
	
	/*!
	 @method     DateFromMonthDayYearHourMinuteSecond (class method)
	 @abstract   Returns an CPDate object representing the date specified by the month, day, year, hour, minute, and second.
	 @discussion You can use this object when you want a specific date and time.
	 @param      Month  The month, identified as a number from 1 to 12.
	 @param      Day  The day of the month.
	 @param      Year  The year, identifed as a complete number (includes century).
	 @param      Hour  The hour, identifed as a number in the range from 0 to 23 (military time).
	 @param      Minute  The minute, identifed as a number in the range from 0 to 59.
	 @param      Second  The second, identifed as a decimal number in the range from 0.000 to 59.999.
	 @result     A CPDate object representing the date and time specified.
	 */
	static CPDate DateFromMonthDayYearHourMinuteSecond(int Month, int Day, int Year, int Hour, int Minute, double Second);
	
	/*!
	 @method     DateFromString (class method)
	 @abstract   Returns an NSDate object with a date or date and time value specified by a given string in the string representation format
	 (MM-DD-YYYY HH:MM:SS) or (MM-DD-YYYY).
	 @discussion You can use this object when you want a specific date and time components.
	 @param      DateString  A string that specifies a date or a date and time value.
	 @result     A CPDate object representing the date and time specified.
	 */
	static CPDate DateFromString(const CPString& DateString);
	
	/*!
	 @method     DateFromStringUsingFormat (class method)
	 @abstract   Returns an NSDate object with a date or date and time value specified by a given string in the format specified.
	 @discussion You can use this object when you want a specific date and time components.
	 @param      DateString  A string that specifies a date or a date and time value.
	 @param      DateFormat  A string that specifies a format that the date string is in.
	 @result     A CPDate object representing the date and time specified.
	 */
	static CPDate DateFromStringUsingFormat(const CPString& DateString, const CPString& DateFormat);
	
	/*!
	 @method     CPDate (constructor)
	 @abstract   Creates an NSDate object that is empty.
	 @discussion This object does not have an underlying date object and as such is limited in functionality. Think of this like having an
	 object pointer that is null.
	 @result     A CPDate object that is empty, it does not represent a date and time.
	 */
	CPDate(void);
	
	/*!
	 @method     CPDate (copy constructor)
	 @abstract   Creates an NSDate object that is the same as the date passed in.
	 @discussion This copy is accomplished by using the same underlying date object as the CPDate passed in.
	 @param      Date  A CPDate to copy.
	 @result     A CPDate object that represent the same date and time as the date passed in.
	 */
	CPDate(const CPDate& Date);
	
	/*!
	 @method     ~CPDate (destructor)
	 @abstract   Deallocates the date object. Removes its reference to its underlying object.
	 @discussion Removes its reference to its underlying object.
	 */
	~CPDate(void);
	
	const CPDate& operator=(const CPDate& Date);
	const CPDate& operator=(CPId Date);
	const CPDate& operator=(const CPString& DateString);
	
	bool operator==(const CPDate& Date) const;			
	bool operator!=(const CPDate& Date) const;
	
	bool operator>(const CPDate& Date) const;
	bool operator<(const CPDate& Date) const;
	bool operator>=(const CPDate& Date) const;
	bool operator<=(const CPDate& Date) const;
	
	void AddDays(int Days);
	void AddTimeInterval(double Seconds);
	
	double TimeIntervalSinceNow(void) const;
	double TimeIntervalSinceRefereceDate(void) const;
	double TimeIntervalSinceDate(CPDate SinceDate) const;

	CPTimeInterval DaysSinceNow(void) const;
	int DaysSinceReferenceDate(void) const;
	
	void SetFromMonthDayYear(int Month, int Day, int Year);
	void SetFromMonthDayYearHourMinuteSecond(int Month, int Day, int Year, int Hour, int Minute, double Second);	

	CPString Description(void) const;
	
#ifdef __OBJC__
#if __COREFOUNDATION__
	CPDate(CFDateRef Date);
	const CPDate& operator=(CFDateRef Date);
	bool operator==(CFDateRef Date) const;
	bool operator!=(CFDateRef Date) const;
	bool operator>(CFDateRef Date) const;
	bool operator<(CFDateRef Date) const;
	bool operator>=(CFDateRef Date) const;
	bool operator<=(CFDateRef Date) const;
	operator CFDateRef(void) const;
#endif  // #if __COREFOUNDATION__
	CPDate(NSDate* Date);
	const CPDate& operator=(NSDate* Date);
	bool operator==(NSDate* Date) const;
	bool operator!=(NSDate* Date) const;
	bool operator>(NSDate* Date) const;
	bool operator<(NSDate* Date) const;
	bool operator>=(NSDate* Date) const;
	bool operator<=(NSDate* Date) const;
	operator NSDate*(void) const;
#endif
	
protected:
	//	CPDate(const CPString& DateString);
	//	CPDate(const CPString& DateString, const CPString& DateFormat);
};
