////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  CPString.h
//    Cross-platform string class.
//
//  Copyright (C) 2009-2010 MYOB US, Inc., all rights reserved.
//
//  Change History (most recent first):
//
//	04/15/10  A.L.L.  Added CStringUsingEncodingWithLossyConversion, to provide lossy conversion between encodings.
//                    Added CPASCIIStringEncoding to support ASCIIString().
//	04/14/10  A.L.L.  Added StringWithCStringUsingEncoding.
//	04/13/10  A.L.L.  Added MacOSRomanString, WindowsLatin1String functions.
//	01/25/10  K.L.S.  Added DoubleToCPString function.
//	01/03/10  K.L.S.  Added constrtuctor and operator= for CPObject.
//	10/20/09  K.L.S.  Created File.
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/CPObject.h"
#include "CrossPlatform/CPStringEncoding.h"
#include "CrossPlatform/CPComparisonResult.h"

#ifdef _DEBUG
//#define DEBUG_STRING
#ifdef DEBUG_STRING
#include <atlstr.h>
#endif
#endif

class CPBool;
class CPArray;
class CPData;
class CPRange;

class EXTERNAL_API CPString : public CPObject
{
public:
	enum Enum_CPStringCompareOptions
	{
		CPCaseInsensitiveSearch = 1,
		CPLiteralSearch = 2,
		CPBackwardsSearch = 4,
		CPAnchoredSearch = 8,
		CPNumericSearch = 64,
	};
	typedef unsigned int CPStringCompareOptions;

	static CPString ClassType(void);
	

	// For formatting info see: http://developer.apple.com/mac/library/documentation/Cocoa/Conceptual/Strings/Articles/formatSpecifiers.html
	static CPString StringWithFormat(const CPString Format,...);
	static CPString StringWithCStringUsingEncoding(const char* String, CPStringEncoding encoding);
	static CPString StringWithCStringUsingEncoding(const unsigned char* String, CPStringEncoding encoding);
//	static CPString StringWithInt(int Value);
//	static CPString StringWIthDouble(double Value);

	CPString(void);
	CPString(const CPString& String);
	CPString(const char* String);
	CPString(const unsigned char* String);
	CPString(const wchar_t* String);
	CPString(const CPData& Data);
	CPString(const CPObject& String);
	CPString(CPId String);

	~CPString(void);

	const CPString& operator=(const CPString& String);
	const CPString& operator=(const char* String);
	const CPString& operator=(const unsigned char* String);
	const CPString& operator=(const wchar_t* String);
	const CPString& operator=(const CPData& Data);
	const CPString& operator=(CPId String);
	const CPString& operator=(const CPObject& String);

	const CPString& operator+=(const CPString& String);
	const CPString& operator+=(const char* String);
	const CPString& operator+=(const unsigned char* String);
	const CPString& operator+=(const wchar_t* String);
	const CPString& operator+=(CPId String);
	//		const CPString& operator+=(CFStringRef String);

	friend CPString EXTERNAL_API operator+(const CPString& String1, const CPString& String2);
	friend CPString EXTERNAL_API operator+(const CPString& String1, const char* String2);
	friend CPString EXTERNAL_API operator+(const char* String1, const CPString& String2);
	friend CPString EXTERNAL_API operator+(const CPString& String1, const unsigned char* String2);
	friend CPString EXTERNAL_API operator+(const unsigned char* String1, const CPString& String2);
	friend CPString EXTERNAL_API operator+(const CPString& String1, const wchar_t* String2);
	friend CPString EXTERNAL_API operator+(const wchar_t* String1, const CPString& String2);
	friend CPString operator+(const CPString& String1, CPId String2);
	friend CPString operator+(CPId String1, const CPString& String2);

	bool operator==(const CPString& String) const;
	bool operator==(const char* String) const;
	bool operator==(const unsigned char* String) const;
	bool operator==(const wchar_t* String) const;
	bool operator==(CPId String) const;

	bool operator!=(const CPString& String) const;
	bool operator!=(const char* String) const;
	bool operator!=(const unsigned char* String) const;
	bool operator!=(const wchar_t* String) const;
	bool operator!=(CPId String) const;

	bool IsEqualToString(const CPString& String) const;
	bool IsEqualToString(const char* String) const;
	bool IsEqualToString(const unsigned char* String) const;
	bool IsEqualToString(const wchar_t* String) const;
	bool IsEqualToString(CPId String) const;
	
	CPComparisonResult Compare(CPString String) const;
	CPComparisonResult CompareWithOptions(CPString String, CPStringCompareOptions CompareOptions) const;
	CPComparisonResult CaseInsensitiveCompare(CPString String) const;
								   
	bool IsEmpty(void) const;
	int Length(void) const;
	CPRange Range(void) const;

	CPRange RangeOfString(const CPString& FindString) const;
	CPRange RangeOfStringWithOptions(const CPString& FindString, CPStringCompareOptions Options) const;
	CPRange RangeOfStringWithOptionsInRange(const CPString& FindString, CPStringCompareOptions Options, const CPRange& SearchRange) const;
	CPString SubstringToIndex(unsigned int To) const;  // string from beginning up to but not including character at index.
	CPString SubstringFromIndex(unsigned int From) const; // string from character at index to the end
	CPString SubstringWithRange(CPRange Range) const; // Substring of string from starting point and length specified

	bool HasPrefix(const CPString& Prefix) const;
	bool HasSuffix(const CPString& Suffix) const;

	CPArray ComponentsSeparatedByString(const CPString& Separator) const;

	// Search and Replace
	CPString StringByReplacingOccurrencesOfStringWithString(const CPString& FindString, const CPString& ReplacementString) const;

	CPData DataContainingStringUsingEncoding(CPStringEncoding Encoding) const;
	
	// CString routines produce CPData objects with a c string terminator added to the end of the contents of the data
	CPData DataContainingCStringUsingEncoding(CPStringEncoding Encoding) const;
	CPData DataContainingUTF8CString(void) const;
	CPData DataContainingASCIICString(void) const;
	CPData DataContainingMacOSRomanCString(void) const;
	CPData DataContainingWindowsLatin1CString(void) const;
	CPData DataContainingUnicodeCString(void) const;

	double DoubleValue(void) const;
	float FloatValue(void) const;
	int IntValue(void) const;
	bool BoolValue(void) const;
	//int IntegerValue(void) const;
	//long long LongLongValue(void) const;

	// Path routines
	CPString StringByAppendingPathComponent(const CPString& pathComponent) const;
	CPString StringByAppendingPathExtension(const CPString& pathComponent) const;
	CPString StringByDeletingLastPathComponent(void) const;
	CPString StringByDeletingPathExtension(void) const;
	CPString LastPathComponent(void) const;
	CPString PathExtension(void) const;

	CPString StringByAddingPercentEscapes(void) const;
	CPString StringByAddingPercentEscapesSpecifyingEscapeCharacters(CPString CharactersToBeEscaped) const;
	
#ifdef __OBJC__
	CPString(NSString* String);
	const CPString& operator=(NSString* String);
	operator NSString*(void) const;
	bool operator==(NSString* String) const;
	bool operator!=(NSString* String) const;

	friend CPString EXTERNAL_API operator+(const CPString& String1, NSString* String2);
	friend CPString EXTERNAL_API operator+(NSString* String1, const CPString& String2);

#endif
#ifdef __COREFOUNDATION_CFBASE__
	CPString(CFStringRef String);
	const CPString& operator=(CFStringRef String);
	operator CFStringRef(void) const;
	bool operator==(CFStringRef String) const;
	bool operator!=(CFStringRef String) const;
	
	friend CPString EXTERNAL_API operator+(const CPString& String1, CFStringRef String2);
	friend CPString EXTERNAL_API operator+(CFStringRef String1, const CPString& String2);
#endif
	
	//#ifdef _WINDOWS
	//		CPString(const CAtlString& String);
	//		const CPString& operator=(const CAtlString& String);
	//		CAtlString ToCString(void) const;
	//		operator CAtlString(void) const;
	//#endif
#ifdef XX__cplusplus_cli
	__stdcall CPString(System::String^ String);
#endif
private:
#ifdef DEBUG_STRING
	CString m_DebugString;
#endif
};

#ifdef __OBJC__
#define CPSTR(s) ((CPString)(@s))
#else
#define CPSTR(s) ((CPString)(s))
#endif
