////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  PDF.h
//      Definition of the PDF class. Class is the encapsulation of a PDF.
//
//  Copyright (C) 2010 MYOB US, Inc., all rights reserved.
//
//  Change History (most recent first):
//
//  08/29/10  K.L.S.  Created file.
//
//  Notes:
//      To use this class the minimium you need to do is the following:
//          PDF pdf;
//          pdf.BeginDocument(PaperRect, PageRect, IsLandscaped); // you can set the page and paper size and orientation if you choose
//      ---- Do the following lines for as many pages as you want
//          pdf.BeginPage();                                      // Starts a new page for output
//          // draw your page
//          pdf.Draw();                                           // Several forms of draw (text, picures, lines, rectangles)
//          pdf.EndPage();
//      ---- End for pages
//          pdf.EndDocument();                                    // Ends the document
//          pdf.Output(OutputFilePath);                           // Full path and name of the file you want to create
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "CrossPlatform/ExternalAPIMacros.h"
#include "CrossPlatform/CPString.h"
#include "CrossPlatform/CPError.h"
#include "CrossPlatform/CPArray.h"
#include "ObsoleteBridge/PDFCoordinate.h"
#include "ObsoleteBridge/PDFPoint.h"
#include "ObsoleteBridge/PDFSize.h"
#include "ObsoleteBridge/PDFRect.h"
#include "ObsoleteBridge/PDFFont.h"
#include "ObsoleteBridge/PDFColor.h"
#include "ObsoleteBridge/PDFImage.h"
#include "ObsoleteBridge/PDFPolygonInfo.h"

namespace ObsoleteBridge
{
	namespace PDFSupport
	{
		class EXTERNAL_API PDF
		{
		public:
			static PDF* Create(void);

			virtual ~PDF(void);
			
			virtual bool ShowPrintDialog(void) = 0;

			virtual CPError BeginDocument(void) = 0;
			virtual CPError BeginDocument(const PDFRect& PageFrame) = 0;
			virtual CPError BeginDocument(const PDFRect& PageFrame, bool IsLandscape) = 0;
			virtual CPError BeginDocument(const PDFRect& PageFrame, const PDFRect& PrintableAreaFrame, bool IsLandscape) = 0;
			virtual CPError EndDocument(void) = 0;
			
			virtual CPError BeginPage(void) = 0;
			virtual CPError EndPage(void) = 0;

			virtual CPError Draw(const PDFRect& Frame, const PDFColor& FrameColor, const PDFColor* FillColor, PDFCoordinate LineWidth) = 0;
			virtual CPError Draw(PDFImage Image, const PDFRect& Frame) = 0;
			virtual CPError Draw(CPString Text, const PDFRect& Frame, const PDFFont& Font, const PDFColor& Color) = 0;
			virtual CPError Draw(const PDFPolygonInfo& PolygonPoints, const PDFColor& FrameColor, const PDFColor* FillColor,
										PDFCoordinate LineWidth) = 0;
			virtual CPError DrawOval(const PDFRect& Frame, const PDFColor& FrameColor, const PDFColor* FillColor,
									 PDFCoordinate LineWidth) = 0;
			virtual CPError DrawRoundedRectangle(const PDFRect&Frame, const PDFColor& FrameColor, const PDFColor* FillColor,
												 PDFCoordinate LineWidth) = 0;

			virtual PDFCoordinate GetLineHeight(void) const = 0;
			virtual PDFCoordinate GetLineHeight(const PDFFont&) const = 0;
			virtual PDFCoordinate GetFontHeight(void) const = 0;
			virtual PDFCoordinate GetFontHeight(const PDFFont&) const = 0;
			virtual PDFCoordinate GetFontHeightWithoutSpacing(void) const = 0;
			virtual PDFCoordinate GetFontHeightWithoutSpacing(const PDFFont&) const = 0;

			virtual PDFCoordinate GetStringWidth(CPString) const = 0;
			virtual PDFCoordinate GetStringWidth(CPString, const PDFFont&) = 0;
	
			virtual CPError Output(CPString) = 0;
			virtual CPError Print(bool ToPreview) = 0;

			virtual void SetDashLine(bool) = 0;
			virtual void SetFont(const PDFFont&) = 0;

			virtual void StartBatchPrinting(void);
			virtual void EndBatchPrinting(void);
			virtual bool IsBatchPrinting(void);
			virtual void SetScalingDisabled(bool DisableScaling);

			//=========================
			//GRW's methods:
			virtual void SetInternalTitle(const CPString& TitleString) = 0;

		protected:
			PDF(void);
		};
	}
}
